<?php

namespace Teknisa\DF\Modules\Entrada\Services;

use Teknisa\DF\Modules\Entrada\Factories\Dfe47000LiberacaoParaFinanceiroFactory;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Teknisa\DF\Modules\Entrada\Services\ManifestacaoDestinatarioService;

class Dfe47000LiberacaoParaFinanceiroService {
    
    protected $Dfe52000Factory;
    protected $Nrorg;
    protected $erros = array();
    protected $titulosSucesso = 0;

    public function __construct(Dfe47000LiberacaoParaFinanceiroFactory $dfe47000LiberacaoParaFinanceiroFactory, $environment, ManifestacaoDestinatarioService $manifestacaoDestinatarioService) {
        $this->dfe47000LiberacaoParaFinanceiroFactory = $dfe47000LiberacaoParaFinanceiroFactory;
        $this->nrorg        = $environment->getNrOrg();
        $this->filialLogada = $environment->getCdFilial();
        $this->environment  = $environment;
        $this->manifestacaoDestinatarioService = $manifestacaoDestinatarioService;
    }

    public function getFilter() {
        try {
            $dados = $this->dfe47000LiberacaoParaFinanceiroFactory->getDadosFilter($this->filialLogada);
            $row   = array( "CDEMPRESA" => $dados["CDEMPRESA"],
                            "NMEMPRESA" => $dados["NMEMPRESA"],
                            "DTENTRSAID" => $this->getDate($dados["DTPROCESSA"]));
        }
        catch(\Exception $e) {
             $row   = array("CDEMPRESA" => "",
                            "NMEMPRESA" => "",
                            "DTENTRSAID" => $this->getDate());
        }
        $row["TCDTIPOOPER"]     = "T";
        $row["TCDSERIEENTR"]    = "T";
        $row["TCDFILIORIG"]     = "T";
        $row["ERRO"]            = "T";
        $row["IDBLOQNFFIN"]     = "T";
        return $row;
    }
    
    public function getDate($dtProcessa = '') {
        if(!$dtProcessa) $dtProcessa = 'now';
        else $dtProcessa = str_replace("/", "-", substr($dtProcessa, 0, 10));
  
        $date   = new \DateTime($dtProcessa);
        $date->modify('last day of this month');
        $mes    = $date->format('m');
        $ano    = $date->format("Y"); 
        $date   = array("start" => "01"."/$mes/".$ano, "end" => $date->format('d/m/Y'));
        return $date;
    }
    
    public function find($conditions) {
        $this->montaInStrField($conditions, array("NMTIPOOPER" => "CDTIPOOPER", "DSSERIEENTR" => "CDSERIEENTR", "NMFILIORIG" => "CDFILIORIG"));
        $conditions = $this->conditionsToArray($conditions);
        $params = $this->equalizeParams($conditions, array("CDTIPOOPER" => "TIPOOPER", "CDFILIORIG" => "FILIAL", "CDEMPRESA" => "EMPRESA",
                                        "CDSERIEENTR" => "SERIE", "IDBLOQNFFIN" => "IDBLOQNFFIN"));
                                        
        $params["EMISINI"] = (isset($conditions["DTEMISSAO"])) ? $conditions["DTEMISSAO"][0] : "30/12/1989";
        $params["EMISFIM"] = (isset($conditions["DTEMISSAO"])) ? $conditions["DTEMISSAO"][1]   : "30/12/1989";
        $params["EMISSAO"] = (!isset($conditions["DTEMISSAO"])) ? 'T' : '';
        $params["ENTRINI"] = $conditions["DTENTRSAID"][0];
        $params["ENTRFIM"] = $conditions["DTENTRSAID"][1];
        
        if(!isset($params["TIPOOPER"]))     $params["TIPOOPER"] = "T";
        if(!isset($params["SERIE"]))        $params["SERIE"] = "T";
        if(!isset($params["FILIAL"]))   $params["FILIAL"] = "T";
        
        if($conditions["ERRO"] == "T") {
            $params["AVISO"] = "T";
            $params["ERRO"]  = "T";
        }
        else if($conditions["ERRO"] == 2) {
            $params["AVISO"] = 0;
            $params["ERRO"]  = 0;
        }
        else {
            $params["AVISO"] = ($params["ERRO"] == 0) ? 1 : 0;
            $params["ERRO"]  = ($params["ERRO"] == 1) ? 1 : 0;
        }
    
        return $this->dfe47000LiberacaoParaFinanceiroFactory->find($params);
    }
    
    private function montaInStrField(&$conditions, $fieldsToInStr) {
        
        foreach($conditions as &$condition) {
            if (array_key_exists($condition["columnName"], $fieldsToInStr)) { 
                $valueField = $condition["value"];
                $sql        = '_';
                foreach($valueField as $value) {
                    $value = (isset($value["CDCENTCUST"])) ? $value["CDCENTCUST"] : $value;
                    $sql.= $value.'_';
                }   
                $novoNome = $fieldsToInStr[$condition["columnName"]];
                $condition["columnName"] = $novoNome;
                $condition["value"] = $sql;
            }
        }
    }
    
    private function conditionsToArray($conditions) {
        $params = array();
        foreach($conditions as $condition) {
            $columnName = $condition["columnName"];
            $params[$columnName] = $condition["value"];
        }
        return $params;
    }
    
    private function equalizeParams($conditions, $relacParams) {
        $params = array();
        foreach($conditions as $columnName => $value) {
            if (array_key_exists($columnName, $relacParams)) { 
                $params[$relacParams[$columnName]] = $value;   
            }
        }
        return $params;
    }
    
     /**
    *   Bloqueia ou libera o titulo de acordo como o parametro $mode
    *   
    *   @param array   $det                 - Dados Completos do produto (impostos, valores, etc)
    *   @param char    $mode                - B (Bloquear) - L (Liberar)
    */
    public function setAprovacao($notas, $mode) {
        $configs        = array();
        $paramManifesta = $this->dfe47000LiberacaoParaFinanceiroFactory->getParamManifesta(); 
        $idManifestacao = ($paramManifesta["IDCIENCIAEMISS"] == '4') ? 4 : ($paramManifesta["IDCONFIRMAEMISS"]) ? 1 : 0;
        $titulosSucesso = 0;
        foreach ($notas as $nota) {
            try {
                if($mode == 'B') {
                    // Rotina para verificação das contas a pagar já geradas (se estão baixadas, etc)
                    $this->dfe47000LiberacaoParaFinanceiroFactory->cancelaVencimento($nota["CDFILIAL"], $nota["NRLANCTONF"]);
                }
                
                if($mode == 'L') {
                    $data = date("d/m/Y H:i:s");
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueado('L', $this->environment->getCdOperador(), '', $data, $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                  
                    // Manifesta a nota de acordo com a parametrização
                    
                    if($idManifestacao != 0) {
                        try {
                            if(!isset($configs[$nota["CDEMPRESA"]])) {
                                $config     = $this->manifestacaoDestinatarioService->montaConfigManifestacao($nota["CDEMPRESA"], $this->environment->getNrOrgTrab(), $nota["CDINSCESTA"]);
                                $configs[$nota["CDEMPRESA"]] = $config;
                            }
                            else {
                                $config = $configs[$nota["CDEMPRESA"]];
                            }
                            $retorno = $this->manifestacaoDestinatarioService->manifestarNfe(array($nota), $config, $idManifestacao);
                        }
                        catch(\Exception $e) {}
                    }
                }
                else {
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueado('B', $this->environment->getCdOperador(), '', '', $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                }
                $titulosSucesso++;
                
            }
            catch(\Exception $e) {
                array_push($this->erros, array("NRNOTAFISC"     => $nota["NRNOTAFISC"],
                                         "DTEMISSAO"            => $nota["DTEMISSAO"],
                                         "VRNOTAFISC"           => $nota["VRNOTAFISC"],
                                         "DSERRO"               => $e->getMessage()));
            }
        }
        return array("SUCESSOS" => $titulosSucesso, "ERROS" => $this->erros);
    }
}